@@ -147,6 +147,15 @@ sub as_sql_having {
         '';
 }
 
+sub as_escape {
+    my ($stmt, $escape_char) = @_;
+
+    # escape_char can be ''(two quotes), or \\ for mysql and \ for others, but it doesn't accept any injections.
+    die 'escape_char length must be up to two characters' if defined($escape_char) && length($escape_char) > 2;
+
+    return " ESCAPE '$escape_char'";
+}
+
 sub add_where {
     my $stmt = shift;
     ## xxx Need to support old range and transform behaviors.
@@ -162,8 +171,10 @@ sub add_complex_where {
     my $stmt = shift;
     my ($terms) = @_;
     my ($where, $bind) = $stmt->_parse_array_terms($terms);
-    push @{ $stmt->{where} }, $where;
-    push @{ $stmt->{bind} }, @$bind;
+    if ($where) {
+        push @{ $stmt->{where} }, $where;
+        push @{ $stmt->{bind} }, @$bind;
+    }
 }
 
 sub _parse_array_terms {
@@ -186,18 +197,22 @@ sub _parse_array_terms {
             foreach my $t2 ( keys %$t ) {
                 my ($term, $bind, $col) = $stmt->_mk_term($t2, $t->{$t2});
                 $stmt->where_values->{$col} = $t->{$t2};
-                push @out, $term;
-                push @bind, @$bind;
+                if ($term) {
+                    push @out, "($term)";
+                    push @bind, @$bind;
+                }
             }
-            $out .= '(' . join(" AND ", @out) . ")";
+            $out .= '(' . join(" AND ", @out) . ")" if @out;
         }
         elsif (ref $t eq 'ARRAY') {
             # another array of terms to process!
             my ($where, $bind) = $stmt->_parse_array_terms( $t );
-            push @bind, @$bind;
-            $out = '(' . $where . ')';
+            if ($where) {
+                push @bind, @$bind;
+                $out = '(' . $where . ')';
+            }
         }
-        push @out, (@out ? ' ' . $logic . ' ' : '') . $out;
+        push @out, (@out ? ' ' . $logic . ' ' : '') . $out if $out;
     }
     return (join("", @out), \@bind);
 }
@@ -270,6 +285,7 @@ sub _mk_term {
                 $term = "$c $val->{op} " . ${$val->{value}};
             } else {
                 $term = "$c $val->{op} ?";
+                $term .= $stmt->as_escape($val->{escape}) if $val->{escape} && $op =~ /^(?:NOT\s+)?I?LIKE$/;
                 push @bind, $val->{value};
             }
         }
